@@ -433,7 +433,7 @@ read_metadata (TrackerResource      *metadata,
 		TrackerResource *white_balance;
 
 		white_balance = tracker_resource_new (md.white_balance);
-		tracker_resource_set_relation (metadata, "nmm:whiteBalance", white_balance);
+		tracker_resource_set_relation (metadata, "nmm:meteringMode", white_balance);
 		g_object_unref (white_balance);
 	}
 
@@ -562,7 +562,7 @@ read_metadata (TrackerResource      *metadata,
 	g_ptr_array_free (keywords, TRUE);
 
 	if (g_strcmp0(pd.software, "gnome-screenshot") == 0) {
-		tracker_resource_add_uri (metadata, "nie:isPartOf", "nfo:image-category-screenshot");
+		tracker_resource_set_string (metadata, "nie:isPartOf", "nfo:image-category-screenshot");
 	}
 
 	tracker_exif_free (ed);
@@ -648,7 +648,7 @@ tracker_extract_get_metadata (TrackerExtractInfo *info)
 
 	info_ptr = png_create_info_struct (png_ptr);
 	if (!info_ptr) {
-		png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
+		png_destroy_read_struct (&png_ptr, NULL, NULL);
 		tracker_file_close (f, FALSE);
 		return FALSE;
 	}
